package com.xhkjedu.slive.service.xz;

import com.xhkjedu.slive.mapper.gradeclass.*;
import com.xhkjedu.slive.mapper.liveplay.CourseClassMapper;
import com.xhkjedu.slive.mapper.liveplay.CourseMapper;
import com.xhkjedu.slive.mapper.liveplay.CourseSectionMapper;
import com.xhkjedu.slive.mapper.xz.XzKjMapper;
import com.xhkjedu.slive.mapper.xz.XzMapper;
import com.xhkjedu.slive.model.gradeclass.TClass;
import com.xhkjedu.slive.model.gradeclass.TClassStudent;
import com.xhkjedu.slive.model.gradeclass.TClassTeacher;
import com.xhkjedu.slive.model.liveplay.LCourse;
import com.xhkjedu.slive.model.liveplay.LCourseClass;
import com.xhkjedu.slive.model.liveplay.LCourseSection;
import com.xhkjedu.slive.model.xz.LXz;
import com.xhkjedu.slive.model.xz.LXzKj;
import com.xhkjedu.slive.service.gradeclass.ClassService;
import com.xhkjedu.slive.utils.LiveUtil;
import com.xhkjedu.slive.vo.gradeclass.ClassVo;
import com.xhkjedu.slive.vo.liveplay.CourseVo;
import com.xhkjedu.slive.vo.xz.XzDateVo;
import com.xhkjedu.slive.vo.xz.XzDateKjTeaVo;
import com.xhkjedu.slive.vo.xz.XzDateKjVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description 行政直播概要
 * @Author WN
 * Date 2022/3/30 13:42
 **/
@Service
@Slf4j
public class XzService {
    @Resource
    private XzMapper xzMapper;
    @Resource
    private XzKjMapper xzKjMapper;
    @Resource
    private CourseMapper courseMapper;
    @Resource
    private CourseSectionMapper courseSectionMapper;
    @Resource
    private CourseClassMapper courseClassMapper;
    @Resource
    private SchoolMapper schoolMapper;
    @Resource
    private ClassService classService;
    @Resource
    private ClassTeacherMapper classTeacherMapper;
    @Resource
    private ClassMapper classMapper;
    @Resource
    private ClassStudentMapper classStudentMapper;
    @Resource
    private GroupMapper groupMapper;
    /**
     * @Description 行政直播列表
     * @Param [schoolid]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/3/30 14:03
     **/
    public List<Map> listXzs(LXz xz) {
        return xzMapper.listXzs(xz);
    }

    /**
     * @Description 保存行政直播
     * @Param [xz]
     * @Return void
     * @Author wn
     * @Date 2022/3/30 14:07
     **/
    @Transactional(rollbackFor = Exception.class)
    public ResultVo saveXz(LXz xz) {
        try {
            Integer timestamp = N_Utils.getSecondTimestamp();
            xz.setCreatetime(timestamp);
            xz.setDeleted(1);
            xzMapper.insertUseGeneratedKeys(xz);//保存行政直播信息
            Integer xzid = xz.getId();
            //保存对应的课节信息
            List<LXzKj> kjlst = new ArrayList<>();
            for (int i = 0; i < xz.getDaynum(); i++) {
                LXzKj kj = new LXzKj();
                kj.setXzid(xzid);
                String kjname = LiveUtil.getChineStrByNum(i);
                kj.setKjname("第" + kjname + "节");
                kj.setKjorder(i + 1);
                kj.setCreateid(xz.getCreateid());
                kj.setCreatetime(timestamp);

                kjlst.add(kj);
            }
            //批量保存行政直播对应的课节
            xzKjMapper.insertList(kjlst);
            return new ResultVo(0, "保存行政直播成功",xzid);
        }catch (Exception e){
            log.error("保存行政直播出错："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new ResultVo(1, "保存行政直播成功失败");
        }
    }

    /**
     * @Description 修改行政直播
     * @Param [xz]
     * @Return void
     * @Author wn
     * @Date 2022/3/30 14:16
     **/
    public void updateXz(LXz xz) {
        xzMapper.updateByPrimaryKeySelective(xz);
    }

    //获取行政直播课下进行中或者已结束的直播课数量
    public Integer getXzSectionKsNum(Integer xzid) {
        Integer num = xzMapper.getXzSectionKsNum(xzid);
        if (num == null) {
            num = 0;
        }
        return num;
    }


    /**
     * @Description 根据行政直播id获取信息
     * @Param [xzid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/4/2 10:41
     **/
    public Map getXzByXzid(Integer xzid) {
        return xzMapper.getXzByXzid(xzid);
    }

    /**
     * @Description 删除行政直播
     * @Param [xzid]
     * @Return void
     * @Author wn
     * @Date 2022/3/30 15:24
     **/
    @Transactional(rollbackFor = Exception.class)
    public void deleteXz(Integer xzid) {
        try {
            //如果行政直播下所有的课都未开始，则可以直接删除，
            //如果有直播课已经开始则进行逻辑删除，并删除所有未开始上的直播课
            Integer sectionnum = xzMapper.getXzSectionKsNum(xzid);
            if (sectionnum != null && sectionnum > 0) {
                //说明直播下有开始或结束的课节，只能删除未开始的直播和直播班级信息
                xzMapper.deleteCourseClassKjByXzid(xzid);
                //修改行政直播的状态未已删除
                xzMapper.updateXzDeleted(10, xzid);
            } else {
                //说明该行政直播课程可以直接删除
                xzMapper.deleteCourseClassByXzid(xzid);
                xzMapper.deleteXzAndKjByXzid(xzid);
            }
        } catch (Exception e) {
            log.error("删除行政直播出错：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * @Description 保存行政直播课程表
     * @Param [xzDatelst]
     * @Return com.xhkjedu.vo.ResultVo
     * @Author wn
     * @Date 2022/3/31 21:46
     **/
    public ResultVo saveXzCourse(LXz xz) {
        try {
            //保存行政直播课程表,参数层级为日期--课节--班级和教师
            //根据同一个课节下根据班级和教师确认是否是同一个直播课，保存进course,section（单课节课程）
            LXz x = xzMapper.selectByPrimaryKey(xz.getXzid());
            List<XzDateVo> xzDatelst = xz.getXzdatas();
            for (int i = 0; i < xzDatelst.size(); i++) {
                //日期下的课节情况
                XzDateVo xzDateVo = xzDatelst.get(i);
                String datazhou = LiveUtil.getDateZhou(xzDateVo.getSelectdate());//要保存日期对应的周
                List<XzDateKjVo> dateKjlst = xzDateVo.getKjs();//指定日期下课节情况

                for (int j = 0; j < dateKjlst.size(); j++) {
                    //单课节情况
                    XzDateKjVo dateKjVo = dateKjlst.get(j);
                    List<XzDateKjTeaVo> xjTealst = dateKjVo.getKjteas();//单课节下各班级情况
                    //处理成教师下班级形式，生成课程
                    Map<Integer, List<XzDateKjTeaVo>> map = listChangeTeacherClass(xjTealst);
                    LCourse course = new LCourse();
                    course.setGradeid(x.getGradeid());
                    course.setBegindate(xzDateVo.getSelectdate());
                    course.setEnddate(xzDateVo.getSelectdate());
                    course.setCoursenum(1);
                    course.setCoursetype(1);
                    course.setCourseclass(1);
                    course.setCoursestate(0);
                    course.setDeleted(1);
                    course.setCourseselected(0);
                    course.setCreateid(xz.getCreateid());
                    course.setCreatetime(N_Utils.getSecondTimestamp());
                    course.setSchoolid(x.getSchoolid());

                    for (Map.Entry<Integer, List<XzDateKjTeaVo>> entry : map.entrySet()) {
                        List<XzDateKjTeaVo> xjTeas = entry.getValue();
                        String teachername = xjTeas.get(0).getTeachername();
                        //单个教师课程信息
                        String coursename =
                                datazhou + dateKjVo.getKjname() + xjTeas.get(0).getSubjectname() + "(" + teachername + ")";

                        course.setCoursename(coursename);
                        course.setSubjectid(xjTeas.get(0).getSubjectid());
                        course.setTeacherid(entry.getKey());
                        course.setBegintime(dateKjVo.getBegintime());
                        course.setEndtime(dateKjVo.getEndtime());
                        course.setKjid(dateKjVo.getKjid());
                        //保存行政直播课程表,该方式解决事务不生效问题
                        ((XzService) AopContext.currentProxy()).saveCourseAll(xjTeas, course);
                    }
                }
            }
        }catch (Exception e){
            log.error("保存行政直播课程表出错："+e.getMessage());
        }

        return new ResultVo(0, "保存行政直播课程表成功");
    }

    //指定天某天某节课下的教师班级层级关系
    private Map<Integer, List<XzDateKjTeaVo>> listChangeTeacherClass(List<XzDateKjTeaVo> xjTealst) {
        Map<Integer, List<XzDateKjTeaVo>> map = new HashMap<>();

        for (int i = 0; i < xjTealst.size(); i++) {
            XzDateKjTeaVo xzDateXjTeaVo = xjTealst.get(i);
            if (!N_Utils.isEmptyInteger(xzDateXjTeaVo.getTeacherid())) {
                if (map.containsKey(xzDateXjTeaVo.getTeacherid())) {
                    map.get(xzDateXjTeaVo.getTeacherid()).add(xzDateXjTeaVo);
                } else {
                    List<XzDateKjTeaVo> classlst = new ArrayList<>();
                    classlst.add(xzDateXjTeaVo);
                    map.put(xzDateXjTeaVo.getTeacherid(), classlst);
                }
            }
        }
        return map;
    }

    //保存行政直播课程表
    @Transactional(rollbackFor = Exception.class)
    public void saveCourseAll(List<XzDateKjTeaVo> xjTeas, LCourse course) {
        try {
            //获取集合中所有的班级id
            List classidlst = xjTeas.stream().map(XzDateKjTeaVo::getClassid).collect(Collectors.toList());
            List<Map> users = courseMapper.listUid(classidlst);//获取课班级下学生

            //新生成直播班班级信息
            TClass c = new TClass();
            c.setClassname(course.getCoursename() + "班");
            c.setCreateid(course.getCreateid());
            c.setSchoolid(course.getSchoolid());
            Integer year = schoolMapper.getYearById(course.getSchoolid());
            c.setSchoolyear(year);
            c.setYear(year);
            c.setClassorder(999);
            c.setGradeid(course.getGradeid());
            c.setClasstype(99);//已直播班级

            Integer classid = classService.save(c);//新生成直播班id
            course.setClassid(classid);

            //设置班级对应教师
            //班级教师
            TClassTeacher teacher = new TClassTeacher();
            teacher.setClassid(classid);
            teacher.setTeacherid(course.getTeacherid());
            teacher.setCreateid(course.getCreateid());
            teacher.setCreatetime(N_Utils.getSecondTimestamp());
            teacher.setSchoolid(course.getSchoolid());
            classTeacherMapper.insertSelective(teacher);

            //保存课程信息
            courseMapper.insertUseGeneratedKeys(course);
            Integer courseid = course.getId();//课程id

            List<Integer> studentids = new ArrayList<>();
            users.forEach(userVo -> {
                studentids.add((Integer) userVo.get("userid"));
            });

            classService.batchSaveStu(course.getCreateid(), course.getSchoolid(), classid, studentids);//批量添加报名学生
            courseMapper.updateStuNum(courseid);//更新已报学生数量

            //保存直播关联班级
            List<LCourseClass> cclist = new ArrayList<>();
            for (int i = 0; i < classidlst.size(); i++) {
                Integer cid = Integer.parseInt(classidlst.get(i).toString());
                LCourseClass cc = new LCourseClass();
                cc.setCourseid(courseid);
                cc.setClassid(cid);
                cc.setCreateid(course.getCreateid());
                cc.setCreatetime(course.getCreatetime());
                cclist.add(cc);
                //通知班级
                ClassVo classVo = new ClassVo();
                classVo.setClassid(cid);
            }
            courseClassMapper.insertList(cclist);//保存原班级和直播的关联关系

            //保存课程课节关联关系
            LCourseSection cs = new LCourseSection();
            cs.setCourseid(courseid);
            cs.setTeacherid(course.getTeacherid());
            cs.setSectionname(course.getCoursename());
            cs.setSectionorder(1);
            cs.setSectiondate(course.getBegindate());
            cs.setPlanbegintime(course.getBegintime());
            cs.setPlanendtime(course.getEndtime());
            cs.setStunum(0);
            cs.setSectionstate(0);
            cs.setCreateid(course.getCreateid());
            cs.setCreatetime(course.getCreatetime());
            courseSectionMapper.insertUseGeneratedKeys(cs);//保存课程课节信息

        } catch (Exception e) {
            log.info("保存行政直播课程表出错：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * @Description 获取行政直播课程表
     * @Param [xz]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/4/1 13:59
     **/
    public Map getXzKjDetail(LXz xz) {
        //获取行政直播信息
        LXz xzobj = xzMapper.selectByPrimaryKey(xz.getXzid());
        xz.setEnddate(xzobj.getEnddate());//放入行政直播实际结束的时间，

        Map map = new HashMap();
        List<XzDateKjVo> kjs = xzKjMapper.listXzKjsByXzid(xz.getXzid());
        map.put("kjs",kjs);//保存所有课节信息

        List<XzDateVo> datelst = new ArrayList<>();//保存指定日期
        //先获取选择要看的有效日期，不能超出行政直播结束的日期
        List<String> seldates = LiveUtil.listDateAfterDates(xz.getBegindate(),xz.getEnddate(),7);
        for(int i=0;i<seldates.size();i++){
            List<XzDateKjVo> kjlst = xzMapper.listXzDateKjDetail(seldates.get(i),xz.getXzid());//获取课节详情
            XzDateVo dateVo = new XzDateVo();
            dateVo.setSelectdate(seldates.get(i));
            dateVo.setKjs(kjlst);
            datelst.add(dateVo);
        }
        map.put("xzdates", datelst);
        return map;
    }

    /**
     * @Description 删除单个单元格
     * @Param [kjTeaVo]
     * @Return com.xhkjedu.vo.ResultVo
     * @Author wn
     * @Date 2022/4/1 17:42
     **/
    public ResultVo delXzkjDyg(XzDateKjTeaVo kjTeaVo) {
        try {
            //判断课程的状态，如果已经开始上课则不能删除
            LCourse course = courseMapper.selectByPrimaryKey(kjTeaVo.getCourseid());
            Integer kjid = course.getKjid();
            Integer classid= kjTeaVo.getClassid();//行政班id
            if(course.getCoursestate() > 0){
                throw new Exception("选择的课程已经开始上课，不允许删除");
            }else{
                //先判断是删除当前课程还是后续日期的课
                if(N_Utils.isEmptyInteger(kjTeaVo.getKfollow())){
                    //kfollow为0或者空，则只需要删除当前选中的课程
                    ((XzService) AopContext.currentProxy()).delXzkjDygForDetail(kjTeaVo,course.getClassid());
                }else{
                    //需要删除后续和当前日期相同的星期相同课节的课程
                    //获取已生成课程的最后日期
                    List<String> alldates = xzMapper.getAllDatesForKj(kjid,classid,course.getBegindate());
                    if(alldates!=null && alldates.size()>0){
                        //已经成课程中符合周的需要删除的日期
                        List<String> deldates = LiveUtil.getMeetDayOfDates(alldates,course.getBegindate());
                        if(deldates!=null && deldates.size()>0){
                            for(int i=0;i<deldates.size();i++){
                                //先获取指定日期对应的课程
                                CourseVo courseVo = courseMapper.getCourseByKjidAndClassid(kjid,classid,deldates.get(i));
                                ((XzService) AopContext.currentProxy()).delXzkjDygForDetail(kjTeaVo,courseVo.getClassid());
                            }
                        }
                    }
                }
                return new ResultVo(0,"成功删除课程表单个课程");
            }
        }catch (Exception e){
            log.error("删除课程表单个课程出错："+e.getMessage());
            return new ResultVo(1,"删除课程表单个课程失败");
        }
    }

    //删除单个课程表
    @Transactional(rollbackFor = Exception.class)
    public void delXzkjDygForDetail(XzDateKjTeaVo kjTeaVo,Integer zbclassid) {
        try {
            //删除单个单元格中设置的教师信息，先判断该单元格对应的课程是否有其他班级
            Integer coursenum = courseClassMapper.getOtherClassnumByCourseid(kjTeaVo.getCourseid(), kjTeaVo.getClassid());
            if (coursenum == 0) {
                //说明该课程仅关联一个班级,直接删除该课程以及该课程对应的直播班
                courseMapper.deleteCourseClassByCourseid(kjTeaVo.getCourseid());
            } else {
                //说明关联的还有其他班级,删除该班级在直播班级中的学生，
                courseClassMapper.deleteByCourseidAndClassid(kjTeaVo.getCourseid(), kjTeaVo.getClassid());
                //处理对应直播班级中学生
                classMapper.deleteClassstu(zbclassid, kjTeaVo.getClassid());
                classMapper.updateNum2(zbclassid);//更改班级以及班级分组人数
                courseMapper.updateStuNum(kjTeaVo.getCourseid());//更新直播课学生数量
            }
        }catch (Exception e){
           log.error("删除单个课程出错："+e.getMessage());
           TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * @Description 修改单个单元格
     * @Param [kjTeaVo]
     * @Return com.xhkjedu.vo.ResultVo
     * @Author wn
     * @Date 2022/4/2 15:19
     **/
    public ResultVo updateXzKjDyg(XzDateKjTeaVo kjTeaVo){
        try {
            //判断课程的状态，如果已经开始上课则不能修改
            LCourse course = courseMapper.selectByPrimaryKey(kjTeaVo.getCourseid());
            Integer kjid = course.getKjid();
            Integer classid= kjTeaVo.getClassid();//行政班id
            if(course.getCoursestate() > 0){
                throw new Exception("选择的课程已经开始上课，不允许修改");
            }else{
                //先判断是修改当前课程还是后续日期的课
                if(N_Utils.isEmptyInteger(kjTeaVo.getKfollow())){
                    //kfollow为0或者空，则只需要修改当前选中的课程
                    ((XzService) AopContext.currentProxy()).updateXzkjDygForDetail(kjTeaVo,course);
                }else{
                    //需要修改后续和当前日期相同的星期相同课节的课程
                    //获取已生成课程的最后日期
                    List<String> alldates = xzMapper.getAllDatesForKj(kjid,classid,course.getBegindate());
                    if(alldates!=null && alldates.size()>0){
                        //已经成课程中符合周的需要修改的日期
                        List<String> deldates = LiveUtil.getMeetDayOfDates(alldates,course.getBegindate());
                        if(deldates!=null && deldates.size()>0){
                            for(int i=0;i<deldates.size();i++){
                                //先获取指定日期对应的课程
                                course.setBegintime(deldates.get(i));
                                ((XzService) AopContext.currentProxy()).updateXzkjDygForDetail(kjTeaVo,course);
                            }
                        }
                    }
                }
                return new ResultVo(0,"成功修改课程表单个课程");
            }
        }catch (Exception e){
            log.error("修改课程表单个课程出错："+e.getMessage());
            return new ResultVo(1,"修改课程表单个课程失败");
        }
    }

    //修改单个课程
    @Transactional(rollbackFor = Exception.class)
    public void updateXzkjDygForDetail(XzDateKjTeaVo kjTeaVo,LCourse course) {
        try {
            //先看新选择的教师情况，先判断选中的同一日期课节教师是否还有其他直播课
            CourseVo courseVo = courseMapper.getCourseByDateKjTeacher(course.getKjid(),kjTeaVo.getTeacherid(),course.getBegindate());
            if(courseVo==null){
                //说明该老师之前没有直播课，则直接修改课程任课教师，直播班关联教师以及科目
                courseMapper.updateCourseSectionClassTeacher(course.getCourseid(),kjTeaVo.getTeacherid(),kjTeaVo.getSubjectid());
            }else{
                //说明该教师当前有其他直播课，则把班级加入课程，完成直播课学生数量
                TClassStudent cs = new TClassStudent();
                cs.setSchoolid(course.getSchoolid());
                cs.setClassid(kjTeaVo.getClassid());
                cs.setCreateid(kjTeaVo.getCreateid());
                classStudentMapper.saveAllClassStu(cs,courseVo.getClassid());
                //获取直播班对应全班学生分组id
                Integer groupid = groupMapper.getGroupIdByClassid(courseVo.getClassid());
                groupMapper.saveAllClassGroupStu(cs,courseVo.getClassid(),groupid);
                classMapper.updateNum2(courseVo.getClassid());//更改直播班级以及班级分组人数
                courseMapper.updateStuNum(courseVo.getCourseid());//更新直播课学生数量

                //把指定的班级从原直播班中移除
                //修改原单元格中设置的教师信息，先判断该单元格对应的课程是否有其他班级
                Integer coursenum = courseClassMapper.getOtherClassnumByCourseid(kjTeaVo.getCourseid(), kjTeaVo.getClassid());
                if (coursenum == 0) {
                    //说明该课程仅关联一个班级,直接修改该课程以及该课程对应的直播班
                    courseMapper.deleteCourseClassByCourseid(kjTeaVo.getCourseid());
                } else {
                    //说明关联的还有其他班级,修改该班级在直播班级中的学生，
                    courseClassMapper.deleteByCourseidAndClassid(kjTeaVo.getCourseid(), kjTeaVo.getClassid());
                    //处理对应直播班级中学生
                    classMapper.deleteClassstu(course.getClassid(), kjTeaVo.getClassid());
                    classMapper.updateNum2(course.getClassid());//更改班级以及班级分组人数
                    courseMapper.updateStuNum(kjTeaVo.getCourseid());//更新直播课学生数量
                }
            }
        }catch (Exception e){
            log.error("修改单个课程出错："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }


}
